package keepwell.fr.agoravoice

import android.Manifest
import android.content.BroadcastReceiver
import android.content.ComponentName
import android.content.ContentValues.TAG
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Bundle
import android.telecom.DisconnectCause
import android.telecom.PhoneAccount
import android.telecom.PhoneAccountHandle
import android.telecom.TelecomManager
import android.util.Log
import androidx.annotation.NonNull
import androidx.core.app.ActivityCompat
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import io.agora.rtc.Constants
import io.agora.rtc.IRtcEngineEventHandler
import io.agora.rtc.RtcEngine
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import org.jetbrains.anko.doAsync
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStream
import java.io.InputStreamReader
import java.io.UnsupportedEncodingException
import java.net.MalformedURLException
import java.net.URL
import java.net.URLConnection


/** AgoravoicePlugin */
class AgoravoicePlugin() : FlutterPlugin, MethodCallHandler {
    private var mRtcEngine: RtcEngine? = null
    private var context: Context? = null

    constructor(context: Context) : this() {
        this.context = context
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        val channel = MethodChannel(flutterPluginBinding.flutterEngine.dartExecutor, "agoravoice")
        channel.setMethodCallHandler(AgoravoicePlugin(flutterPluginBinding.applicationContext));
        val event = EventChannel(flutterPluginBinding.flutterEngine.dartExecutor, "listener")
        event.setStreamHandler(object : EventChannel.StreamHandler {
            override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
                mRtcEventHandler = object : IRtcEngineEventHandler() {
                    override fun onUserJoined(uid: Int, elapsed: Int) {
                        data.postValue("user joined")
                        SoundPoolManager.getInstance(context).stopRinging()
                    }

                    override fun onUserOffline(uid: Int, reason: Int) {
                        data.postValue("user left")
                    }
                }

                data.observeForever(Observer {
                    events?.success(it)
                })
            }

            override fun onCancel(arguments: Any?) {
            }
        })
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {

        val data: MutableLiveData<String> by lazy {
            MutableLiveData<String>()
        }
        lateinit var mRtcEventHandler: IRtcEngineEventHandler
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "agoravoice")
            channel.setMethodCallHandler(AgoravoicePlugin(registrar.activeContext()))

            val event = EventChannel(registrar.messenger(), "listener")
            event.setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
                    mRtcEventHandler = object : IRtcEngineEventHandler() {
                        override fun onUserJoined(uid: Int, elapsed: Int) {
                            data.postValue("user joined")
                            SoundPoolManager.getInstance(registrar.activeContext()).stopRinging()
                        }

                        override fun onUserOffline(uid: Int, reason: Int) {
                            data.postValue("user left")
                        }
                    }

                    data.observeForever(Observer {
                        events?.success(it)
                    })
                }

                override fun onCancel(arguments: Any?) {
                }
            })
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            "init" -> {
                val appId = call.argument<String>("appId") ?: ""
                initializeAgoraEngine(appId)
                result.success(true)
            }
            "join" -> {
                val accessTokenUrl = call.argument<String>("accessTokenUrl") ?: ""
                val channel = call.argument<String>("channel") ?: ""
                doAsync {
                    joinChannel(getContent("$accessTokenUrl?channel=$channel"), channel)
                }
                result.success(true)
            }
            "leave" -> {
                leaveChannel()
                result.success(true)
            }

            "speaker" -> {
                val enable = call.argument<Boolean>("enable") ?: false
                enableSpeakerPhone(enable)
            }

            "mute" -> {
                val mute = call.argument<Boolean>("mute") ?: false
                muteCall(mute)
            }

            "dispose" -> {
                dispose()
            }

            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    }

    private fun initializeAgoraEngine(appId: String) {
        try {
            SoundPoolManager.getInstance(context).playRinging()
            mRtcEngine = RtcEngine.create(context, appId, mRtcEventHandler)
            mRtcEngine?.setChannelProfile(Constants.CHANNEL_PROFILE_COMMUNICATION)

        } catch (e: Exception) {
            //Log.e(io.agora.tutorials1v1acall.VoiceChatViewActivity.LOG_TAG, Log.getStackTraceString(e))
            throw RuntimeException("NEED TO check rtc sdk init fatal error\n" + Log.getStackTraceString(e))
        }
    }

    private fun joinChannel(accessToken: String, channel: String) {
        mRtcEngine?.joinChannel(accessToken, channel, "", 0) // if you do not specify the uid, we will generate the uid for you
    }

    // Tutorial Step 3
    private fun leaveChannel() {
        SoundPoolManager.getInstance(context).stopRinging()
        SoundPoolManager.getInstance(context).playDisconnect()
        mRtcEngine?.leaveChannel()
    }

    private fun enableSpeakerPhone(enable: Boolean) {
        mRtcEngine?.setEnableSpeakerphone(enable)
    }

    private fun muteCall(muted: Boolean) {
        mRtcEngine?.muteLocalAudioStream(muted)
    }

    private fun dispose() {
        leaveChannel()
        RtcEngine.destroy()
        SoundPoolManager.getInstance(context).release()
        mRtcEngine = null
    }

    fun getContent(url: String?): String {
        var contents: String? = ""
        try {
            val conn: URLConnection = URL(url).openConnection()
            val input: InputStream = conn.getInputStream()
            contents = convertStreamToString(input)
        } catch (e: MalformedURLException) {
        } catch (e: IOException) {
        }
        return contents ?: ""
    }

    @Throws(UnsupportedEncodingException::class)
    private fun convertStreamToString(inputStream: InputStream): String? {
        val reader = BufferedReader(InputStreamReader(inputStream, "UTF-8"))
        val sb = StringBuilder()
        var line: String? = null
        try {
            while (reader.readLine().also { line = it } != null) {
                sb.append(line + "n")
            }
        } catch (e: IOException) {
            e.printStackTrace()
        } finally {
            try {
                inputStream.close()
            } catch (e: IOException) {
                e.printStackTrace()
            }
        }
        return sb.toString()
    }
}

